package cn.efunbox.manage.common.utils;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang3.StringUtils;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import sun.misc.BASE64Decoder;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import java.io.IOException;
import java.security.*;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.X509EncodedKeySpec;

/**
 * RSA加解密方法 RSA生成公钥和私钥 RSA从文件中读取公钥和私钥 RSA从字符串中读取公钥和私钥
 *
 * @author guixin
 */
@SuppressWarnings("restriction")
public class RSAEncrypt {

    private static final String DEFAULT_PUBLIC_KEY = "MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQClXd7uSEQEPAMI842Qvt3vD6oa"
            + "\r"
            + "nuJk+bKMSs/AwpmIkyjfy1xeK1C9P5XfDxFXlXUWkClhX0yCiKEdeSWf6XgwtZX9"
            + "\r"
            + "8//2mb79FhBTZUgFerRNFEQc66RbgClnD3sNg8Svdil281UIz9EuDPQ5dRRdajmj"
            + "\r" + "dAGIPGrIZVIZHtybYwIDAQAB" + "\r";

    // 字节数据转字符串专用集合
    private static final char[] HEX_CHAR = {'0', '1', '2', '3', '4', '5', '6',
            '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};

    private final static BouncyCastleProvider bcp = new BouncyCastleProvider();
    private RSAPublicKey publicKey;// 自动生成的公钥

    public RSAPublicKey getPublicKey() {
        return publicKey;
    }

    /**
     * 随机生成密钥对
     *
     * @throws NoSuchAlgorithmException
     */
    public void genKeyPair() throws NoSuchAlgorithmException {
        KeyPairGenerator keyPairGen = KeyPairGenerator.getInstance("RSA");
        keyPairGen.initialize(1024, new SecureRandom());
        KeyPair keyPair = keyPairGen.generateKeyPair();
        this.publicKey = (RSAPublicKey) keyPair.getPublic();
    }

    /**
     * 加密过程
     *
     * @param publicKey     公钥
     * @param plainTextData 明文数据
     * @return
     * @throws Exception 加密过程中的异常信息
     */
    public static byte[] encrypt(RSAPublicKey publicKey, byte[] plainTextData)
            throws Exception {
        if (publicKey == null) {
            throw new Exception("加密公钥为空, 请设置");
        }

        Cipher cipher = null;
        try {
            cipher = Cipher.getInstance("RSA", bcp);
            cipher.init(Cipher.ENCRYPT_MODE, publicKey);
            return cipher.doFinal(plainTextData);
        } catch (NoSuchAlgorithmException e) {
            throw new Exception("无此加密算法");
        } catch (NoSuchPaddingException e) {
            e.printStackTrace();
            return null;
        } catch (InvalidKeyException e) {
            throw new Exception("加密公钥非法,请检查");
        } catch (IllegalBlockSizeException e) {
            throw new Exception("明文长度非法");
        } catch (BadPaddingException e) {
            throw new Exception("明文数据已损坏");
        }
    }

    /**
     * 字节数据转十六进制字符串
     *
     * @param data 输入数据
     * @return 十六进制内容
     */
    public static String byteArrayToString(byte[] data) {
        StringBuilder stringBuilder = new StringBuilder();
        for (int i = 0; i < data.length; i++) {
            stringBuilder.append(HEX_CHAR[(data[i] & 0xf0) >>> 4]);
            stringBuilder.append(HEX_CHAR[(data[i] & 0x0f)]);
            if (i < data.length - 1) {
                stringBuilder.append(' ');
            }
        }
        return stringBuilder.toString();
    }

    /**
     * 从字符串中加载公钥
     *
     * @param publicKeyStr 公钥数据字符串
     * @throws Exception 加载公钥时产生的异常
     */
    public static RSAPublicKey loadPublicKeyByStr(String publicKeyStr)
            throws Exception {
        try {
            BASE64Decoder base64Decoder = new BASE64Decoder();
            byte[] buffer = base64Decoder.decodeBuffer(publicKeyStr);
            KeyFactory keyFactory = KeyFactory.getInstance("RSA");
            X509EncodedKeySpec keySpec = new X509EncodedKeySpec(buffer);
            return (RSAPublicKey) keyFactory.generatePublic(keySpec);
        } catch (NoSuchAlgorithmException e) {
            throw new Exception("无此算法");
        } catch (InvalidKeySpecException e) {
            throw new Exception("公钥非法");
        } catch (IOException e) {
            throw new Exception("公钥数据内容读取错误");
        } catch (NullPointerException e) {
            throw new Exception("公钥数据为空");
        }
    }

    /**
     * 对字符串进行加密处理
     *
     * @param plainTextData
     * @return
     * @throws Exception
     */
    public static String efunoxRSAEncrypt(String plainTextData) {
        if (StringUtils.isNotBlank(plainTextData)) {
            RSAPublicKey publicKey = null;
            try {
                publicKey = RSAEncrypt
                        .loadPublicKeyByStr(RSAEncrypt.DEFAULT_PUBLIC_KEY);

                byte[] mi = RSAEncrypt.encrypt(publicKey, plainTextData.getBytes());
                return Base64.encodeBase64String(mi);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    public static void main(String[] args) throws Exception {
        // 测试字符串
        String plainTextData = "Test String chaijunkun";
        String str = efunoxRSAEncrypt(plainTextData);
        System.out.println(str);
    }
}
